"""
  Lists.py -- some useful and fast stuff for list-handling (Version 0.7)

  * in most of theses functions, condition has to be a function,
    taking one argument and returning either 1 or 0 (representing true
    and false) (in many cases, None serves as identity-function)

  * most functions work for tuples too

  * Any suggestions for completion or improvement of this module ? I'm still
    looking for the fastest implementations possible.

  XXX: use operator-module more extensively

  Thanks to everybody who contributed so far:
    Guido van Rossum (Python and sortedby),
    Skip Montanaro (forall and exists),
    ??? (list),
    Martin v. Loewis (exists, count, init, id_uniques)

    
-----------------------------------------------------------------------------
(c) Copyright by Marc-Andre Lemburg (mailto:mal@lemburg.com)

    Permission to use, copy, modify, and distribute this software and its
    documentation for any purpose and without fee or royalty is hereby granted,
    provided that the above copyright notice appear in all copies and that
    both that copyright notice and this permission notice appear in
    supporting documentation or portions thereof, including modifications,
    that you make.

    THE AUTHOR MARC-ANDRE LEMBURG DISCLAIMS ALL WARRANTIES WITH REGARD TO
    THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
    FITNESS, IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL,
    INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
    FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
    NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
    WITH THE USE OR PERFORMANCE OF THIS SOFTWARE !


"""

__version__ = '0.7'

import sys

# for backward compatibility
if sys.version[:3] >= '1.4': 
    import operator
    __and__ = operator.__and__
    __or__ = operator.__or__
    __add__ = operator.__add__
else:
    __and__ = lambda x,y: x and y
    __or__ = lambda x,y: x or y
    __add__ = lambda x,y: x+y

def exists(condition,list):

    """
        exists: return 1 iff condition evaluates to 1 for at least one element in list,
                       0 otherwise
    """
    return reduce(lambda x,y,c=condition: x or c(y), list, 0)

def forall(condition,list):

    """
        forall: return 1 iff condition evaluates to 1 for every element in list,
                       0 otherwise
    """
    return reduce( __and__, map(condition, list), 1)

def count(condition,list):

    """
        count: count the number of elements in list, for which condition eval. to 1
       
    """
    return len(filter(condition,list))

def weighted_sum(weightfct,list):

    """
        weighted_sum: sum up the elements of list, using weights (which weightfct has
                      to return) for each element

        e.g.
         weighted_sum(None,range(1,100)) does, what the teacher had in mind, when
                                         he gave Gauss this problem...
    """
    return reduce( __add__, map(weightfct, list), 0)

def cut(list1,list2):

    """
        cut: implements a set-cut operation for lists; it returns a list of
             elements in list1 (not copies !), that are common to both lists,
             excluding doubles.

        e.g.
         cut([1,2,3],[2.0,3.0,4.0]) gives [2, 3]
                          
        (only works for hashable list elements !)
    """
    d = {}
    for x in list2:
        d[x] = 0
    l = []
    for x in list1:
        if d.has_key(x) and not d[x]:
            l.append(x)
            d[x] = 1
    return l

def join(list1,list2):

    """
        join: implements a set-joincut operation for lists; it returns a list of
              elements (not copies !) that are common to both lists, excluding
              doubles. Elements in list1 have precedence over the ones in list2.
              The resulting list is unordered with respect to the two input lists.

        e.g.
         join([1,2,3],[2.0,3.0,4.0]) gives [4.0, 3, 2, 1]
                          
        (only works for hashable list elements !)
    """
    d = {}
    for x in list1: d[x] = 0
    for x in list2: d[x] = 0
    return d.keys()

def zip(*lists):

    """
        zip: zip lists together forming a list of tuples
       
        e.g.
         zip([1,2,3],[4,5,6]) returns [(1, 4), (2, 5), (3, 6)]

        * note: there is a special case to this function, when one or more lists
                differ in size: None is then taken as 'entry'

        (map already does this, when called with None as function !)
    """
    return apply(map,(None,)+lists)

def unzip(list):

    """
        unzip: inverse of zip - converts a list of tuples into a tuple of lists
       
        e.g.
         a,b = unzip(zip(a,b))

        * note: all tuples in list have to have the same length, if not,
                this function will fail

    """
    if len(list) == 0: return ()
    l = []
    for t in range(len(list[0])):
        l.append(map( lambda x,t=t: x[t], list ))
    return tuple(l)

def indexfilter(condition,list):

    """
        indexfilter: returns a list consisting only of those elements
                     for which condition(element-index) == 1

         e.g. 
          indexfilter( lambda i: i%2==0, list) will return every second element starting
                                               from the first

        (standard stuff, just here for completeness)
    """
    l = []
    for i in range(len(list)):
        if condition(i): l.append(list[i])
    return l

def extract(indices,list):

    """
        extract: returns a list consisting only of the elements indexed by
                 indices

        In Python1.5 with my list.get-patch this gives the same
        result: map(list.get,indices).

        BTW: this works for dictionaries too (I love Python !-)... just give
             the function different parameters:

        extract(keys,dict):
                 returns a list consisting only of the corresponding values
                 of keys taken from dict

        (standard stuff, just here for completeness)
    """
    l = [None] * len(indices)
    for i in range(len(indices)):
        l[i] = list[indices[i]]
    return l

def sortedby(list,*indices):

    """
        sortedby: sort a list of lists (e.g. a table) by one or more indices
                  (columns of the table) and return the sorted list

        e.g.
         for list = [[2,3],[1,2],[3,1]]:
         sortedby(list,1) will return [[3, 1], [1, 2], [2, 3]],
         sortedby(list,0) will return [[1, 2], [2, 3], [3, 1]]
    """

    nlist = map( lambda x,indices=indices: 
                 map( lambda i,x=x: x[i], indices) + [x],
                 list)
    nlist.sort()
    return map( lambda l: l[-1], nlist)

def uniques(list):

    """
        uniques: returns a list of unique values in list
       
        (only works for hashable list elements !)
    """
    d = {}
    for x in list: d[x] = 0
    return d.keys()

def id_uniques(list):
    
    """
       id_uniques: returns a list of unique entries (via id(), not values!) in list

       (works for all types of list elements: unique elements are identified
        by their identity, rather than their value !)
    """
    d = {}
    for x in list: d[id(x)]=x
    return d.values()

def items2dict(dict_items):

    """
       items2dict: inverse of dict.items()

       (XXX not exactly the right place in this module... should probably be moved
        to a dicts - module of the same kind)
    """
    d = {}
    for x,y in dictitems:
        d[x] = y
    return d

# try to use the available C functions instead if possible
try:
    from lists import *
except:
    pass

# tests

if __name__ == '__main__':

    import time
    t=time.time()
    l = range(100000)
    print 'Starting tests...',
    try:
        loop(1,int)
        print 'using C functions'
    except:
        print 'using Python functions'
    print
    print 'exists',;t=time.time()
    k = exists( lambda x: x>50000,l)
    print time.time()-t,"seconds"
    print 'forall',;t=time.time()
    k = forall( lambda x: x<=50000,l)
    print time.time()-t,"seconds"
    print 'count',;t=time.time()
    k = count( lambda x: x>50000,l)
    print time.time()-t,"seconds"
    print 'weighted_sum',;t=time.time()
    k = weighted_sum( lambda x: 0.1*x, l)
    print time.time()-t,"seconds"
    print 'indexfilter ',;t=time.time()
    k = indexfilter( lambda x:x%5==0, l)
    print time.time()-t,"seconds"
    print 'extract',;t=time.time()
    k = extract(range(1,100000,3), l)
    print time.time()-t,"seconds"
    print 'zip',;t=time.time()
    k = zip(l, l)
    print time.time()-t,"seconds"
    print 'unzip',;t=time.time()
    m = unzip(k)
    print time.time()-t,"seconds"
    print 'uniques',;t=time.time()
    k = uniques(l)
    print time.time()-t,"seconds"
    print 'Finished.'
